/**
 * Pyx4me framework
 * Copyright (C) 2006-2008 pyx4j.com.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 * 
 * @author vlads
 * @version $Id: AbstractDeviceMojo.java 2066 2008-06-21 23:16:18Z vlads $
 */
package com.pyx4me.maven.obex;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.settings.Server;
import org.apache.maven.settings.Settings;
import org.codehaus.plexus.util.xml.Xpp3Dom;

public abstract class AbstractDeviceMojo extends AbstractMojo {
	/**
	 * The server id prefix in settings.xml to use when connecting.
	 * 
	 * @parameter expression="${obex.server}" default-value="obex-device"
	 * @required
	 */
	protected String server = "obex-device";

	/**
	 * Specifies optional sufix for server ID to use.
	 * 
	 * @parameter expression="${obex.c}"
	 */
	protected String serverSufix;

	/**
	 * The current user system settings for use in Maven.
	 * 
	 * @parameter expression="${settings}"
	 * @required
	 * @readonly
	 */
	protected Settings settings;

	private static boolean configured = false;

	protected ServerConfig getServerConfig(String serverSufix) throws MojoFailureException {
		ServerConfig cfg = new ServerConfig();

		cfg.serverID = server;
		if (serverSufix != null) {
			cfg.serverID += serverSufix;
		}

		Server srv = settings.getServer(cfg.serverID);
		if (srv == null) {
			throw new MojoFailureException("You have to configure the server with the id '" + cfg.serverID
					+ "' in your seetings.xml file.");
		}
		Xpp3Dom config = (Xpp3Dom) srv.getConfiguration();
		Xpp3Dom url = config.getChild("url");
		if (url == null) {
			throw new MojoFailureException("You have to configure the server '" + cfg.serverID
					+ "'  configuration url in your seetings.xml file.");
		}
		cfg.url = url.getValue();
		cfg.alias = cfg.serverID + " (" + url.getValue() + ")";
		Xpp3Dom aliasXml = config.getChild("alias");
		if (aliasXml != null) {
			cfg.alias = aliasXml.getValue();
		}
		Xpp3Dom stackXml = config.getChild("stack");
		if (stackXml != null) {
			cfg.stack = stackXml.getValue();
		}
		return cfg;
	}

	static void configureBlueCove(ServerConfig cfg, Log log) {
		if (configured) {
			return;
		}
		configured = true;
		try {
			BlueCoveSpecific.configureBlueCove(cfg, log);
		} catch (Throwable notBlueCove) {
		}
	}
}
